@@ -323,8 +323,6 @@ static int __init cops_probe1(struct net_device *dev, int ioaddr)
 			break;
 	}
 
-	dev->base_addr = ioaddr;
-
 	/* Reserve any actual interrupt. */
 	if (dev->irq) {
 		retval = request_irq(dev->irq, cops_interrupt, 0, dev->name, dev);
@@ -332,6 +330,8 @@ static int __init cops_probe1(struct net_device *dev, int ioaddr)
 			goto err_out;
 	}
 
+	dev->base_addr = ioaddr;
+
         lp = netdev_priv(dev);
         spin_lock_init(&lp->lock);
 
@@ -860,7 +860,7 @@ static void cops_timeout(struct net_device *dev)
 	}
 	printk(KERN_WARNING "%s: Transmit timed out.\n", dev->name);
 	cops_jumpstart(dev);	/* Restart the card. */
-	dev->trans_start = jiffies; /* prevent tx timeout */
+	netif_trans_update(dev); /* prevent tx timeout */
 	netif_wake_queue(dev);
 }
 
