@@ -604,10 +604,6 @@ static int firm_send_command(struct usb_serial_port *port, __u8 command,
 
 	command_port = port->serial->port[COMMAND_PORT];
 	command_info = usb_get_serial_port_data(command_port);
-
-	if (command_port->bulk_out_size < datasize + 1)
-		return -EIO;
-
 	mutex_lock(&command_info->mutex);
 	command_info->command_finished = false;
 
@@ -681,7 +677,6 @@ static void firm_setup_port(struct tty_struct *tty)
 	struct device *dev = &port->dev;
 	struct whiteheat_port_settings port_settings;
 	unsigned int cflag = tty->termios.c_cflag;
-	speed_t baud;
 
 	port_settings.port = port->port_number + 1;
 
@@ -742,13 +737,11 @@ static void firm_setup_port(struct tty_struct *tty)
 	dev_dbg(dev, "%s - XON = %2x, XOFF = %2x\n", __func__, port_settings.xon, port_settings.xoff);
 
 	/* get the baud rate wanted */
-	baud = tty_get_baud_rate(tty);
-	port_settings.baud = cpu_to_le32(baud);
-	dev_dbg(dev, "%s - baud rate = %u\n", __func__, baud);
+	port_settings.baud = tty_get_baud_rate(tty);
+	dev_dbg(dev, "%s - baud rate = %d\n", __func__, port_settings.baud);
 
 	/* fixme: should set validated settings */
-	tty_encode_baud_rate(tty, baud, baud);
-
+	tty_encode_baud_rate(tty, port_settings.baud, port_settings.baud);
 	/* handle any settings that aren't specified in the tty structure */
 	port_settings.lloop = 0;
 
